@@ -1,6 +1,6 @@
 package Win32;
 
-BEGIN {
+# BEGIN {
     use strict;
     use vars qw|$VERSION $XS_VERSION @ISA @EXPORT @EXPORT_OK|;
 
@@ -8,7 +8,7 @@ BEGIN {
     require DynaLoader;
 
     @ISA = qw|Exporter DynaLoader|;
-    $VERSION = '0.37';
+    $VERSION = '0.40';
     $XS_VERSION = $VERSION;
     $VERSION = eval $VERSION;
 
@@ -79,7 +79,7 @@ BEGIN {
         CSIDL_RESOURCES_LOCALIZED
         CSIDL_CDBURN_AREA
     );
-}
+# }
 
 # We won't bother with the constant stuff, too much of a hassle.  Just hard
 # code it here.
@@ -154,89 +154,403 @@ sub CSIDL_RESOURCES            ()       { 0x0038 }     # %windir%\Resources\, Fo
 sub CSIDL_RESOURCES_LOCALIZED  ()       { 0x0039 }     # %windir%\Resources\<LangID>, for theme and other windows specific resources.
 sub CSIDL_CDBURN_AREA          ()       { 0x003B }     # <user name>\Local Settings\Application Data\Microsoft\CD Burning
 
+sub VER_NT_DOMAIN_CONTROLLER () { 0x0000002 } # The system is a domain controller and the operating system is Windows Server 2008, Windows Server 2003, or Windows 2000 Server.
+sub VER_NT_SERVER () { 0x0000003 } # The operating system is Windows Server 2008, Windows Server 2003, or Windows 2000 Server.
+# Note that a server that is also a domain controller is reported as VER_NT_DOMAIN_CONTROLLER, not VER_NT_SERVER.
+sub VER_NT_WORKSTATION () { 0x0000001 } # The operating system is Windows Vista, Windows XP Professional, Windows XP Home Edition, or Windows 2000 Professional.
+
+
+sub VER_SUITE_BACKOFFICE               () { 0x00000004 } # Microsoft BackOffice components are installed.
+sub VER_SUITE_BLADE                    () { 0x00000400 } # Windows Server 2003, Web Edition is installed.
+sub VER_SUITE_COMPUTE_SERVER           () { 0x00004000 } # Windows Server 2003, Compute Cluster Edition is installed.
+sub VER_SUITE_DATACENTER               () { 0x00000080 } # Windows Server 2008 Datacenter, Windows Server 2003, Datacenter Edition, or Windows 2000 Datacenter Server is installed.
+sub VER_SUITE_ENTERPRISE               () { 0x00000002 } # Windows Server 2008 Enterprise, Windows Server 2003, Enterprise Edition, or Windows 2000 Advanced Server is installed. Refer to the Remarks section for more information about this bit flag.
+sub VER_SUITE_EMBEDDEDNT               () { 0x00000040 } # Windows XP Embedded is installed.
+sub VER_SUITE_PERSONAL                 () { 0x00000200 } # Windows Vista Home Premium, Windows Vista Home Basic, or Windows XP Home Edition is installed.
+sub VER_SUITE_SINGLEUSERTS             () { 0x00000100 } # Remote Desktop is supported, but only one interactive session is supported. This value is set unless the system is running in application server mode.
+sub VER_SUITE_SMALLBUSINESS            () { 0x00000001 } # Microsoft Small Business Server was once installed on the system, but may have been upgraded to another version of Windows. Refer to the Remarks section for more information about this bit flag.
+sub VER_SUITE_SMALLBUSINESS_RESTRICTED () { 0x00000020 } # Microsoft Small Business Server is installed with the restrictive client license in force. Refer to the Remarks section for more information about this bit flag.
+sub VER_SUITE_STORAGE_SERVER           () { 0x00002000 } # Windows Storage Server 2003 R2 or Windows Storage Server 2003 is installed.
+sub VER_SUITE_TERMINAL                 () { 0x00000010 } # Terminal Services is installed. This value is always set.
+# If VER_SUITE_TERMINAL is set but VER_SUITE_SINGLEUSERTS is not set, the system is running in application server mode.
+sub VER_SUITE_WH_SERVER                () { 0x00008000 } # Windows Home Server is installed.
+
+
+sub SM_TABLETPC                ()       { 86 }
+sub SM_MEDIACENTER             ()       { 87 }
+sub SM_STARTER                 ()       { 88 }
+sub SM_SERVERR2                ()       { 89 }
+
+sub PRODUCT_UNDEFINED                        () { 0x000 } # An unknown product
+sub PRODUCT_ULTIMATE                         () { 0x001 } # Ultimate
+sub PRODUCT_HOME_BASIC                       () { 0x002 } # Home Basic
+sub PRODUCT_HOME_PREMIUM                     () { 0x003 } # Home Premium
+sub PRODUCT_ENTERPRISE                       () { 0x004 } # Enterprise
+sub PRODUCT_HOME_BASIC_N                     () { 0x005 } # Home Basic N
+sub PRODUCT_BUSINESS                         () { 0x006 } # Business
+sub PRODUCT_STANDARD_SERVER                  () { 0x007 } # Server Standard (full installation)
+sub PRODUCT_DATACENTER_SERVER                () { 0x008 } # Server Datacenter (full installation)
+sub PRODUCT_SMALLBUSINESS_SERVER             () { 0x009 } # Windows Small Business Server
+sub PRODUCT_ENTERPRISE_SERVER                () { 0x00A } # Server Enterprise (full installation)
+sub PRODUCT_STARTER                          () { 0x00B } # Starter
+sub PRODUCT_DATACENTER_SERVER_CORE           () { 0x00C } # Server Datacenter (core installation)
+sub PRODUCT_STANDARD_SERVER_CORE             () { 0x00D } # Server Standard (core installation)
+sub PRODUCT_ENTERPRISE_SERVER_CORE           () { 0x00E } # Server Enterprise (core installation)
+sub PRODUCT_ENTERPRISE_SERVER_IA64           () { 0x00F } # Server Enterprise for Itanium-based Systems
+sub PRODUCT_BUSINESS_N                       () { 0x010 } # Business N
+sub PRODUCT_WEB_SERVER                       () { 0x011 } # Web Server (full installation)
+sub PRODUCT_CLUSTER_SERVER                   () { 0x012 } # HPC Edition
+sub PRODUCT_HOME_SERVER                      () { 0x013 } # Home Server Edition
+sub PRODUCT_STORAGE_EXPRESS_SERVER           () { 0x014 } # Storage Server Express
+sub PRODUCT_STORAGE_STANDARD_SERVER          () { 0x015 } # Storage Server Standard
+sub PRODUCT_STORAGE_WORKGROUP_SERVER         () { 0x016 } # Storage Server Workgroup
+sub PRODUCT_STORAGE_ENTERPRISE_SERVER        () { 0x017 } # Storage Server Enterprise
+sub PRODUCT_SERVER_FOR_SMALLBUSINESS         () { 0x018 } # Windows Server 2008 for Windows Essential Server Solutions
+sub PRODUCT_SMALLBUSINESS_SERVER_PREMIUM     () { 0x019 } # Windows Small Business Server Premium
+sub PRODUCT_HOME_PREMIUM_N                   () { 0x01A } # Home Premium N
+sub PRODUCT_ENTERPRISE_N                     () { 0x01B } # Enterprise N
+sub PRODUCT_ULTIMATE_N                       () { 0x01C } # Ultimate N
+sub PRODUCT_WEB_SERVER_CORE                  () { 0x01D } # Web Server (core installation)
+sub PRODUCT_MEDIUMBUSINESS_SERVER_MANAGEMENT () { 0x01E } # Windows Essential Business Server Management Server
+sub PRODUCT_MEDIUMBUSINESS_SERVER_SECURITY   () { 0x01F } # Windows Essential Business Server Security Server
+sub PRODUCT_MEDIUMBUSINESS_SERVER_MESSAGING  () { 0x020 } # Windows Essential Business Server Messaging Server
+sub PRODUCT_SERVER_FOUNDATION                () { 0x021 } # Server Foundation
+
+sub PRODUCT_SERVER_FOR_SMALLBUSINESS_V       () { 0x023 } # Windows Server 2008 without Hyper-V for Windows Essential Server Solutions
+sub PRODUCT_STANDARD_SERVER_V                () { 0x024 } # Server Standard without Hyper-V (full installation)
+sub PRODUCT_DATACENTER_SERVER_V              () { 0x025 } # Server Datacenter without Hyper-V (full installation)
+sub PRODUCT_ENTERPRISE_SERVER_V              () { 0x026 } # Server Enterprise without Hyper-V (full installation)
+sub PRODUCT_DATACENTER_SERVER_CORE_V         () { 0x027 } # Server Datacenter without Hyper-V (core installation)
+sub PRODUCT_STANDARD_SERVER_CORE_V           () { 0x028 } # Server Standard without Hyper-V (core installation)
+sub PRODUCT_ENTERPRISE_SERVER_CORE_V         () { 0x029 } # Server Enterprise without Hyper-V (core installation)
+sub PRODUCT_HYPERV                           () { 0x02A } # Microsoft Hyper-V Server
+
+sub PRODUCT_STARTER_N                        () { 0x02F } # Starter N
+sub PRODUCT_PROFESSIONAL                     () { 0x030 } # Professional
+sub PRODUCT_PROFESSIONAL_N                   () { 0x031 } # Professional N
+
+sub PRODUCT_STARTER_E                        () { 0x042 } # Starter E
+sub PRODUCT_HOME_BASIC_E                     () { 0x043 } # Home Basic E
+sub PRODUCT_HOME_PREMIUM_E                   () { 0x044 } # Home Premium E
+sub PRODUCT_PROFESSIONAL_E                   () { 0x045 } # Professional E
+sub PRODUCT_ENTERPRISE_E                     () { 0x046 } # Enterprise E
+sub PRODUCT_ULTIMATE_E                       () { 0x047 } # Ultimate E
+
+sub PRODUCT_UNLICENSED                       () { 0xABCDABCD } # product has not been activated and is no longer in the grace period
+
+sub PROCESSOR_ARCHITECTURE_AMD64   ()   { 9 }      # x64 (AMD or Intel)
+sub PROCESSOR_ARCHITECTURE_IA64    ()   { 6 }      # Intel Itanium Processor Family (IPF)
+sub PROCESSOR_ARCHITECTURE_INTEL   ()   { 0 }      # x86
+sub PROCESSOR_ARCHITECTURE_UNKNOWN ()   { 0xffff } # Unknown architecture.
+
+sub _GetProcessorArchitecture {
+    my $arch = {
+	 386 => PROCESSOR_ARCHITECTURE_INTEL,
+	 486 => PROCESSOR_ARCHITECTURE_INTEL,
+	 586 => PROCESSOR_ARCHITECTURE_INTEL,
+	2200 => PROCESSOR_ARCHITECTURE_IA64,
+	8664 => PROCESSOR_ARCHITECTURE_AMD64,
+    }->{Win32::GetChipName()};
+    return defined($arch) ? $arch : PROCESSOR_ARCHITECTURE_UNKNOWN;
+}
+
 ### This method is just a simple interface into GetOSVersion().  More
 ### specific or demanding situations should use that instead.
 
-my ($found_os, $found_desc);
+my ($cached_os, $cached_desc);
 
 sub GetOSName {
-    my ($os,$desc,$major, $minor, $build, $id)=("","");
-    unless (defined $found_os) {
-        # If we have a run this already, we have the results cached
-        # If so, return them
+    unless (defined $cached_os) {
+	my($desc, $major, $minor, $build, $id, undef, undef, $suitemask, $producttype)
+	    = Win32::GetOSVersion();
+	my $arch = _GetProcessorArchitecture();
+	my $productinfo = Win32::GetProductInfo(6, 0, 0, 0);
+	($cached_os, $cached_desc) = _GetOSName($desc, $major, $minor, $build, $id,
+						$suitemask, $producttype, $productinfo, $arch);
+    }
+    return wantarray ? ($cached_os, $cached_desc) : $cached_os;
+}
 
-        # Use the standard API call to determine the version
-        ($desc, $major, $minor, $build, $id) = Win32::GetOSVersion();
+sub GetOSDisplayName {
+    # Calling GetOSDisplayName() with arguments is for the test suite only!
+    my($name,$desc) = @_ ? @_ : GetOSName();
+    $name =~ s/^Win//;
+    if ($desc eq "Windows Home Server" || $desc eq "Windows XP Professional x64 Edition") {
+	($name, $desc) = ($desc, "");
+    }
+    elsif ($desc =~ s/\s*(Windows (.*) Server( \d+)?)$//) {
+       ($name, $desc) = ("$1 $name", $desc);
+    }
+    else {
+	for ($name) {
+	    s/^/Windows / unless /^Win32s$/;
+	    s/\/.Net//;
+	    s/NT(\d)/NT $1/;
+	    if ($desc =~ s/\s*(HPC|Small Business|Web) Server//) {
+		my $name = $1;
+		$desc =~ s/^\s*//;
+		s/(200.)/$name Server $1/;
+	    }
+	    s/^Windows (200[38])/Windows Server $1/;
+	}
+    }
+    $name .= " $desc" if length $desc;
+    return $name;
+}
 
-        # If id==0 then its a win32s box -- Meaning Win3.11
-        unless($id) {
-            $os = 'Win32s';
-        }
-	else {
-	    # Magic numbers from MSDN documentation of OSVERSIONINFO
-	    # Most version names can be parsed from just the id and minor
-	    # version
-	    $os = {
-		1 => {
-		    0  => "95",
-		    10 => "98",
-		    90 => "Me"
-		},
-		2 => {
-		    0  => "NT4",
-		    1  => "XP/.Net",
-		    2  => "2003",
-		    51 => "NT3.51"
+sub _GetSystemMetrics {
+    my($index,$metrics) = @_;
+    return $metrics->{$index} if ref $metrics eq "HASH" && defined $metrics->{$index};
+    return 1 if ref $metrics eq "ARRAY" && grep $_ == $index, @$metrics;
+    return Win32::GetSystemMetrics($index);
+}
+
+sub _GetOSName {
+    # The $metrics argument only exists for the benefit of t/GetOSName.t
+    my($csd, $major, $minor, $build, $id, $suitemask, $producttype, $productinfo, $arch, $metrics) = @_;
+
+    my($os,@tags);
+    my $desc = "";
+    if ($id == 0) {
+	$os = "Win32s";
+    }
+    elsif ($id == 1) {
+	if ($minor == 0) {
+	    $os = "95";
+	}
+	elsif ($minor == 10) {
+	    $os = "98";
+	}
+	elsif ($minor == 90) {
+	    $os = "Me";
+	}
+    }
+    elsif ($id == 2) {
+	if ($major == 3) {
+	    $os = "NT3.51";
+	}
+	elsif ($major == 4) {
+	    $os = "NT4";
+	}
+	elsif ($major == 5) {
+	    if ($minor == 0) {
+		$os = "2000";
+		if ($producttype == VER_NT_WORKSTATION) {
+		    $desc = "Professional";
 		}
-	    }->{$id}->{$minor};
-	}
-
-        # This _really_ shouldnt happen.  At least not for quite a while
-        # Politely warn and return undef
-        unless (defined $os) {
-            warn qq[Windows version [$id:$major:$minor] unknown!];
-            return undef;
-        }
-
-        my $tag = "";
-
-        # But distinguising W2k and Vista from NT4 requires looking at the major version
-        if ($os eq "NT4") {
-	    $os = {5 => "2000", 6 => "Vista"}->{$major} || "NT4";
-        }
-
-        # For the rest we take a look at the build numbers and try to deduce
-	# the exact release name, but we put that in the $desc
-        elsif ($os eq "95") {
-            if ($build eq '67109814') {
-                    $tag = '(a)';
-            }
-	    elsif ($build eq '67306684') {
-                    $tag = '(b1)';
-            }
-	    elsif ($build eq '67109975') {
-                    $tag = '(b2)';
-            }
-        }
-	elsif ($os eq "98" && $build eq '67766446') {
-            $tag = '(2nd ed)';
-        }
-
-	if (length $tag) {
-	    if (length $desc) {
-	        $desc = "$tag $desc";
+		else {
+		    if ($suitemask & VER_SUITE_DATACENTER) {
+			$desc = "Datacenter Server";
+		    }
+		    elsif ($suitemask & VER_SUITE_ENTERPRISE) {
+			$desc = "Advanced Server";
+		    }
+		    elsif ($suitemask & VER_SUITE_SMALLBUSINESS_RESTRICTED) {
+			$desc = "Small Business Server";
+		    }
+		    else {
+			$desc = "Server";
+		    }
+		}
+		# XXX ignoring "Windows 2000 Advanced Server Limited Edition" for Itanium
+		# XXX and "Windows 2000 Datacenter Server Limited Edition" for Itanium
 	    }
-	    else {
-	        $desc = $tag;
+	    elsif ($minor == 1) {
+		$os = "XP/.Net";
+		if (_GetSystemMetrics(SM_MEDIACENTER, $metrics)) {
+		    $desc = "Media Center Edition";
+		}
+		elsif (_GetSystemMetrics(SM_TABLETPC, $metrics)) {
+		    # Tablet PC Edition is based on XP Pro
+		    $desc = "Tablet PC Edition";
+		}
+		elsif (_GetSystemMetrics(SM_STARTER, $metrics)) {
+		    $desc = "Starter Edition";
+		}
+		elsif ($suitemask & VER_SUITE_PERSONAL) {
+		    $desc = "Home Edition";
+		}
+		else {
+		    $desc = "Professional";
+		}
+		# XXX ignoring all Windows XP Embedded and Fundamentals versions
+	    }
+	    elsif ($minor == 2) {
+		$os = "2003";
+
+		if (_GetSystemMetrics(SM_SERVERR2, $metrics)) {
+		    # XXX R2 was released for all x86 and x64 versions,
+		    # XXX but only Enterprise Edition for Itanium.
+		    $desc = "R2";
+		}
+
+		if ($suitemask == VER_SUITE_STORAGE_SERVER) {
+		    $desc .= " Windows Storage Server";
+		}
+		elsif ($suitemask == VER_SUITE_WH_SERVER) {
+		    $desc .= " Windows Home Server";
+		}
+		elsif ($producttype == VER_NT_WORKSTATION && $arch == PROCESSOR_ARCHITECTURE_AMD64) {
+		    $desc .= " Windows XP Professional x64 Edition";
+		}
+
+		# Test for the server type.
+		if ($producttype != VER_NT_WORKSTATION) {
+		    if ($arch == PROCESSOR_ARCHITECTURE_IA64) {
+			if ($suitemask & VER_SUITE_DATACENTER) {
+			    $desc .= " Datacenter Edition for Itanium-based Systems";
+			}
+			elsif ($suitemask & VER_SUITE_ENTERPRISE) {
+			    $desc .= " Enterprise Edition for Itanium-based Systems";
+			}
+		    }
+		    elsif ($arch == PROCESSOR_ARCHITECTURE_AMD64) {
+			if ($suitemask & VER_SUITE_DATACENTER) {
+			    $desc .= " Datacenter x64 Edition";
+			}
+			elsif ($suitemask & VER_SUITE_ENTERPRISE) {
+			    $desc .= " Enterprise x64 Edition";
+			}
+			else {
+			    $desc .= " Standard x64 Edition";
+			}
+		    }
+		    else {
+			if ($suitemask & VER_SUITE_COMPUTE_SERVER) {
+			    $desc .= " Windows Compute Cluster Server";
+			}
+			elsif ($suitemask & VER_SUITE_DATACENTER) {
+			    $desc .= " Datacenter Edition";
+			}
+			elsif ($suitemask & VER_SUITE_ENTERPRISE) {
+			    $desc .= " Enterprise Edition";
+			}
+			elsif ($suitemask & VER_SUITE_BLADE) {
+			    $desc .= " Web Edition";
+			}
+			elsif ($suitemask & VER_SUITE_SMALLBUSINESS_RESTRICTED) {
+			    $desc .= " Small Business Server";
+			}
+			else {
+			    if ($desc !~ /Windows (Home|Storage) Server/) {
+				$desc .= " Standard Edition";
+			    }
+			}
+		    }
+		}
 	    }
 	}
+	elsif ($major == 6) {
+	    if ($minor == 0) {
+		if ($producttype == VER_NT_WORKSTATION) {
+		    $os = "Vista";
+		}
+		else {
+		    $os = "2008";
+		}
+	    }
+	    elsif ($minor == 1) {
+		if ($producttype == VER_NT_WORKSTATION) {
+		    $os = "7";
+		}
+		else {
+		    $os = "2008";
+		    $desc = "R2";
+		}
+	    }
 
-        # cache the results, so we dont have to do this again
-        $found_os      = "Win$os";
-        $found_desc    = $desc;
+            if ($productinfo == PRODUCT_ULTIMATE) {
+		$desc .= " Ultimate";
+	    }
+            elsif ($productinfo == PRODUCT_HOME_PREMIUM) {
+               $desc .= " Home Premium";
+            }
+            elsif ($productinfo == PRODUCT_HOME_BASIC) {
+               $desc .= " Home Basic";
+            }
+            elsif ($productinfo == PRODUCT_ENTERPRISE) {
+               $desc .= " Enterprise";
+            }
+            elsif ($productinfo == PRODUCT_BUSINESS) {
+               $desc .= " Business";
+            }
+            elsif ($productinfo == PRODUCT_STARTER) {
+               $desc .= " Starter";
+            }
+            elsif ($productinfo == PRODUCT_CLUSTER_SERVER) {
+               $desc .= " HPC Server";
+            }
+            elsif ($productinfo == PRODUCT_DATACENTER_SERVER) {
+               $desc .= " Datacenter";
+            }
+            elsif ($productinfo == PRODUCT_DATACENTER_SERVER_CORE) {
+               $desc .= " Datacenter Edition (core installation)";
+            }
+            elsif ($productinfo == PRODUCT_ENTERPRISE_SERVER) {
+               $desc .= " Enterprise";
+            }
+            elsif ($productinfo == PRODUCT_ENTERPRISE_SERVER_CORE) {
+               $desc .= " Enterprise Edition (core installation)";
+            }
+            elsif ($productinfo == PRODUCT_ENTERPRISE_SERVER_IA64) {
+               $desc .= " Enterprise Edition for Itanium-based Systems";
+            }
+            elsif ($productinfo == PRODUCT_SMALLBUSINESS_SERVER) {
+               $desc .= " Small Business Server";
+            }
+            elsif ($productinfo == PRODUCT_SMALLBUSINESS_SERVER_PREMIUM) {
+               $desc .= " Small Business Server Premium Edition";
+            }
+            elsif ($productinfo == PRODUCT_STANDARD_SERVER) {
+               $desc .= " Standard";
+            }
+            elsif ($productinfo == PRODUCT_STANDARD_SERVER_CORE) {
+               $desc .= " Standard Edition (core installation)";
+            }
+            elsif ($productinfo == PRODUCT_WEB_SERVER) {
+               $desc .= " Web Server";
+            }
+            elsif ($productinfo == PRODUCT_PROFESSIONAL) {
+               $desc .= " Professional";
+            }
+
+	    if ($arch == PROCESSOR_ARCHITECTURE_INTEL) {
+		$desc .= " (32-bit)";
+	    }
+	    elsif ($arch == PROCESSOR_ARCHITECTURE_AMD64) {
+		$desc .= " (64-bit)";
+	    }
+	}
     }
 
-    return wantarray ? ($found_os, $found_desc) : $found_os;
+    unless (defined $os) {
+	warn "Unknown Windows version [$id:$major:$minor]";
+	return;
+    }
+
+    for ($desc) {
+	s/\s\s+/ /g;
+	s/^\s//;
+	s/\s$//;
+    }
+
+    # XXX What about "Small Business Server"? NT, 200, 2003, 2008 editions...
+
+    if ($major >= 5) {
+	# XXX XP, Vista, 7 all have starter editions
+	#push(@tags, "Starter Edition") if _GetSystemMetrics(SM_STARTER, $metrics);
+    }
+
+    if (@tags) {
+	unshift(@tags, $desc) if length $desc;
+	$desc = join(" ", @tags);
+    }
+
+    if (length $csd) {
+	$desc .= " " if length $desc;
+	$desc .= $csd;
+    }
+    return ("Win$os", $desc);
 }
 
 # "no warnings 'redefine';" doesn't work for 5.8.7 and earlier
@@ -390,8 +704,8 @@ $ENV{PROCESSOR_ARCHITECTURE}.  This might not work on Win9X.
 
 =item Win32::GetChipName()
 
-Returns the processor type: 386, 486 or 586 for Intel processors,
-21064 for the Alpha chip.
+Returns the processor type: 386, 486 or 586 for x86 processors,
+8664 for the x64 processor and 2200 for the Itanium.
 
 =item Win32::GetCwd()
 
@@ -529,6 +843,68 @@ before passing the path to a system call or another program.
 [CORE] Returns a string in the form of "<d>:" where <d> is the first
 available drive letter.
 
+=item Win32::GetOSDisplayName()
+
+Returns the "marketing" name of the Windows operating system version
+being used.  It returns names like these (random samples):
+
+   Windows 2000 Datacenter Server
+   Windows XP Professional
+   Windows XP Tablet PC Edition
+   Windows Home Server
+   Windows Server 2003 Enterprise Edition for Itanium-based Systems
+   Windows Vista Ultimate (32-bit)
+   Windows Small Business Server 2008 R2 (64-bit)
+
+This function should only be used to display the actual OS name to the
+user; it should not be used to determine the class of operating systems
+this system belongs to.  The Win32::GetOSName(), Win32::GetOSVersion,
+Win32::GetProductInfo() and Win32::GetSystemMetrics() functions provide
+the base information to check for certain capabilities, or for families
+of OS releases.
+
+=item Win32::GetOSName()
+
+In scalar context returns the name of the Win32 operating system
+being used.  In list context returns a two element list of the OS name
+and whatever edition information is known about the particular build
+(for Win9X boxes) and whatever service packs have been installed.
+The latter is roughly equivalent to the first item returned by
+GetOSVersion() in list context.
+
+The description will also include tags for other special editions,
+like "R2", "Media Center", "Tablet PC", or "Starter Edition".
+
+Currently the possible values for the OS name are
+
+    WinWin32s
+    Win95
+    Win98
+    WinMe
+    WinNT3.51
+    WinNT4
+    Win2000
+    WinXP/.Net
+    Win2003
+    WinHomeSvr
+    WinVista
+    Win2008
+    Win7
+
+This routine is just a simple interface into GetOSVersion().  More
+specific or demanding situations should use that instead.  Another
+option would be to use POSIX::uname(), however the latter appears to
+report only the OS family name and not the specific OS.  In scalar
+context it returns just the ID.
+
+The name "WinXP/.Net" is used for historical reasons only, to maintain
+backwards compatibility of the Win32 module.  Windows .NET Server has
+been renamed as Windows 2003 Server before final release and uses a
+different major/minor version number than Windows XP.
+
+Similarly the name "WinWin32s" should have been "Win32s" but has been
+kept as-is for backwards compatibility reasons too.
+
 =item Win32::GetOSVersion()
 
 [CORE] Returns the list (STRING, MAJOR, MINOR, BUILD, ID), where the
@@ -536,25 +912,46 @@ elements are, respectively: An arbitrary descriptive string, the major
 version number of the operating system, the minor version number, the
 build number, and a digit indicating the actual operating system.
 For the ID, the values are 0 for Win32s, 1 for Windows 9X/Me and 2 for
-Windows NT/2000/XP/2003/Vista.  In scalar context it returns just the ID.
+Windows NT/2000/XP/2003/Vista/2008/7.  In scalar context it returns just
+the ID.
 
 Currently known values for ID MAJOR and MINOR are as follows:
 
-    OS                    ID    MAJOR   MINOR
-    Win32s                 0      -       -
-    Windows 95             1      4       0
-    Windows 98             1      4      10
-    Windows Me             1      4      90
-    Windows NT 3.51        2      3      51
-    Windows NT 4           2      4       0
-    Windows 2000           2      5       0
-    Windows XP             2      5       1
-    Windows Server 2003    2      5       2
-    Windows Vista          2      6       0
+    OS                      ID    MAJOR   MINOR
+    Win32s                   0      -       -
+    Windows 95               1      4       0
+    Windows 98               1      4      10
+    Windows Me               1      4      90
+
+    Windows NT 3.51          2      3      51
+    Windows NT 4             2      4       0
+
+    Windows 2000             2      5       0
+    Windows XP               2      5       1
+    Windows Server 2003      2      5       2
+    Windows Server 2003 R2   2      5       2
+    Windows Home Server      2      5       2
+
+    Windows Vista            2      6       0
+    Windows Server 2008      2      6       0
+    Windows 7                2      6       1
+    Windows Server 2008 R2   2      6       1
 
 On Windows NT 4 SP6 and later this function returns the following
 additional values: SPMAJOR, SPMINOR, SUITEMASK, PRODUCTTYPE.
 
+The version numbers for Windows 2003 and Windows Home Server are
+identical; the SUITEMASK field must be used to differentiate between\
+them.
+
+The version numbers for Windows Vista and Windows Server 2008 are
+identical; the PRODUCTTYPE field must be used to differentiate between
+them.
+
+The version numbers for Windows 7 and Windows Server 2008 R2 are
+identical; the PRODUCTTYPE field must be used to differentiate between
+them.
+
 SPMAJOR and SPMINOR are are the version numbers of the latest
 installed service pack.
 
@@ -574,6 +971,9 @@ the system.  Known bits are:
     VER_SUITE_BLADE                     0x00000400
     VER_SUITE_EMBEDDED_RESTRICTED       0x00000800
     VER_SUITE_SECURITY_APPLIANCE        0x00001000
+    VER_SUITE_STORAGE_SERVER            0x00002000
+    VER_SUITE_COMPUTE_SERVER            0x00004000
+    VER_SUITE_WH_SERVER                 0x00008000
 
 The VER_SUITE_xxx names are listed here to crossreference the Microsoft
 documentation.  The Win32 module does not provide symbolic names for these
@@ -584,31 +984,10 @@ be one of the following integer values:
 
     1 - Workstation (NT 4, 2000 Pro, XP Home, XP Pro, Vista)
     2 - Domaincontroller
-    3 - Server
+    3 - Server (2000 Server, Server 2003, Server 2008)
 
-=item Win32::GetOSName()
-
-In scalar context returns the name of the Win32 operating system
-being used.  In list context returns a two element list of the OS name
-and whatever edition information is known about the particular build
-(for Win9X boxes) and whatever service packs have been installed.
-The latter is roughly equivalent to the first item returned by
-GetOSVersion() in list context.
-
-Currently the possible values for the OS name are
-
- Win32s Win95 Win98 WinMe WinNT3.51 WinNT4 Win2000 WinXP/.Net Win2003 WinVista
-
-This routine is just a simple interface into GetOSVersion().  More
-specific or demanding situations should use that instead.  Another
-option would be to use POSIX::uname(), however the latter appears to
-report only the OS family name and not the specific OS.  In scalar
-context it returns just the ID.
-
-The name "WinXP/.Net" is used for historical reasons only, to maintain
-backwards compatibility of the Win32 module.  Windows .NET Server has
-been renamed as Windows 2003 Server before final release and uses a
-different major/minor version number than Windows XP.
+Note that a server that is also a domain controller is reported as
+PRODUCTTYPE 2 (Domaincontroller) and not PRODUCTTYPE 3 (Server).
 
 =item Win32::GetShortPathName(PATHNAME)
 
@@ -620,6 +999,13 @@ path containing spaces.  Returns C<undef> when the PATHNAME does not
 exist. Compare with Win32::GetFullPathName() and
 Win32::GetLongPathName().
 
+=item Win32::GetSystemMetrics(INDEX)
+
+Retrieves the specified system metric or system configuration setting.
+Please refer to the Microsoft documentation of the GetSystemMetrics()
+function for a reference of available INDEX values.  All system
+metrics return integer values.
+
 =item Win32::GetProcAddress(INSTANCE, PROCNAME)
 
 Returns the address of a function inside a loaded library.  The
@@ -627,6 +1013,19 @@ information about what you can do with this address has been lost in
 the mist of time.  Use the Win32::API module instead of this deprecated
 function.
 
+=item Win32::GetProductInfo(OSMAJOR, OSMINOR, SPMAJOR, SPMINOR)
+
+Retrieves the product type for the operating system on the local
+computer, and maps the type to the product types supported by the
+specified operating system.  Please refer to the Microsoft
+documentation of the GetProductInfo() function for more information
+about the parameters and return value.  This function requires Windows
+Vista or later.
+
+See also the Win32::GetOSName() and Win32::GetOSDisplayName()
+functions which provide a higher level abstraction of the data
+returned by this function.
+
 =item Win32::GetTickCount()
 
 [CORE] Returns the number of milliseconds elapsed since the last
@@ -645,7 +1044,7 @@ The return value is formatted according to OLE conventions, as groups
 of hex digits with surrounding braces.  For example:
 
     {09531CF1-D0C7-4860-840C-1C8C8735E2AD}
- 
+
 =item Win32::InitiateSystemShutdown
 
 (MACHINE, MESSAGE, TIMEOUT, FORCECLOSE, REBOOT)
